-- AISMod.lua
-- author: MonoBlack
-- date: 13.11.2024
-- Copyright 2024

AISMod = {}
local AISMod_mt = Class(AISMod)

g_isAISModDeveloper = false
g_hasAISPerformedDebug = false
AISMod.hasPrintedError = false  -- Variable, um Fehlermeldungen zu kontrollieren

AISMod.ENABLE_CONSOLE_COMMANDS_INTERFACE = true
AISMod.ENABLE_AIS = true

function AISMod:new(isClient, modDir)
	if g_aisMod ~= nil then
		print("AISMod: Instanz existiert bereits.")
		return
	end

	local self = {}
	setmetatable(self, AISMod_mt)

	self.baseDir = modDir
	self.isClient = isClient

	addModEventListener(self)
	print("AISMod: Neue Instanz erstellt.")

	return self
end

function AISMod:onLoad()
	if self.isClient then
		print("AISMod: Drivable geladen.")
		resetSteers(self)
	end
end

function AISMod:onUpdate(dt, isActiveForInput, isActiveForInputIgnoreSelection, isSelected)
	if self.isClient and AISMod.ENABLE_AIS then
		
		local drivable_isActive = self:getIsActive() and isActiveForInputIgnoreSelection;
    
		--															for avoiding error with steering
		if self.spec_drivable ~= nil and drivable_isActive and self.spec_drivable.steeringWheel ~= nil then
			local steerRot = getSteeringRotation(self.spec_drivable)

			-- place for debuging tables to find functions which are not in docs
			if g_isAISModDeveloper ~= nil and g_isAISModDeveloper and not g_hasAISPerformedDebug then
				DebugUtil.printTableRecursively(self.spec_drivable.steeringWheel);
				g_hasAISPerformedDebug = true
			end

			local isSteering = self.spec_drivable.hasSteerToTurnRightDirection or self.spec_drivable.hasSteerToTurnLeftDirection;

			if isSteering then
				-- mod checks for permmission if it could automaticly turn off this lights 
				-- if the steering wheel is nearly centered 
				--				- you cloud customize these numbers i decided to set these as defaults
				if self.spec_drivable.hasSteerToTurnRightDirection and steerRot >= -0.001 then
					self:setTurnLightState(Lights.TURNLIGHT_OFF)
					resetSteers(self)
				elseif self.spec_drivable.hasSteerToTurnLeftDirection and steerRot <= 0.001 then
					self:setTurnLightState(Lights.TURNLIGHT_OFF)
					resetSteers(self)
				end
			-- check turn dricections
			elseif self.spec_lights.turnLightState == Lights.TURNLIGHT_RIGHT then
				-- register custom attr for chekcs above on next update
				self.spec_drivable.hasSteerToTurnRightDirection = steerRot < -0.1;
			elseif self.spec_lights.turnLightState == Lights.TURNLIGHT_LEFT then
				-- register custom attr for chekcs above on next update
				self.spec_drivable.hasSteerToTurnLeftDirection = steerRot > 0.1;
			end
    	end
	end
end;	

function getSteeringRotation(spec_drivable)
	if spec_drivable and spec_drivable.steeringWheel and spec_drivable.steeringWheel.node then
		local _, rotation, _ = getRotation(spec_drivable.steeringWheel.node)
		return rotation
	else
		if not AISMod.hasPrintedError then
			print("AISMod: Lenkradknoten konnte nicht gefunden werden.")
			AISMod.hasPrintedError = true -- Fehlermeldung nur einmal ausgeben
		end
		return 0
	end
end

function initAISMod()
	if g_aisMod == nil and g_currentModName == "FS25_AutoTurnSignalOff" then
		local aisMod = AISMod:new(g_dedicatedServerInfo == nil, g_currentModDirectory)
		
		if aisMod then
			getfenv(0)["g_aisMod"] = aisMod

			if aisMod.isClient and AISMod.ENABLE_CONSOLE_COMMANDS_INTERFACE then
				source(g_currentModDirectory .. "src/ConsoleCommands.lua")
				aisMod.consoleCommands = ConsoleCommands:new(aisMod)
			end

			print("AISMod: Mod erfolgreich geladen.")
		else
			print("AISMod: Mod konnte nicht initialisiert werden.")
		end
	else
		print("AISMod: Bereits initialisiert oder falscher Mod-Name.")
	end
end

function AISMod:loadMap(name)
	Drivable.onLoad = Utils.prependedFunction(Drivable.onLoad, AISMod.onLoad)
	Drivable.onUpdate = Utils.prependedFunction(Drivable.onUpdate, AISMod.onUpdate)
end

function AISMod:deleteMap()
	if self.consoleCommands ~= nil then
		self.consoleCommands:delete()
	end

	local env = getfenv(0)
	env["g_aisMod"] = nil
	getfenv(0)["g_aisMod"] = nil
end

function delete()
	AISMod:deleteMap()
end

function resetSteers(modSpec)
	modSpec.spec_drivable.hasSteerToTurnLeftDirection = false
	modSpec.spec_drivable.hasSteerToTurnRightDirection = false
end

-- Mod initialisieren
initAISMod()
