-- ConsoleCommands.lua
-- author: MonoBlack
-- date: 13.11.2024
-- Copyright 2024

ConsoleCommands = {}
local ConsoleCommands_mt = Class(ConsoleCommands)

function ConsoleCommands:new(parent)
	if g_aisMod == nil then
		print("AISMod: Fehler - Mod nicht gefunden.")
		return
	end

	local self = setmetatable({}, ConsoleCommands_mt)
	self.parent = parent  -- Referenz auf das Elternmodul, um Zugriff auf AISMod zu erleichtern

	-- Konsolenbefehle hinzufügen
	addConsoleCommand("activateAIS", "Aktiviert den automatischen Blinker-Stop", "consoleCommandActivateAIS", self)
	addConsoleCommand("deactivateAIS", "Deaktiviert den automatischen Blinker-Stop", "consoleCommandDeactivateAIS", self)
	addConsoleCommand("getAISStatus", "Zeigt den aktuellen Status des AIS-Systems an", "consoleCommandStatus", self)

	return self
end

function ConsoleCommands:delete()
	-- Konsolenbefehle entfernen
	removeConsoleCommand("activateAIS")
	removeConsoleCommand("deactivateAIS")
	removeConsoleCommand("getAISStatus")

	-- Wenn die Hauptmod-Instanz vorhanden ist, setze consoleCommands auf nil
	if g_aisMod ~= nil then
		g_aisMod.consoleCommands = nil
	end
end

function ConsoleCommands:consoleCommandActivateAIS()
	if not AISMod.ENABLE_AIS then
		AISMod.ENABLE_AIS = true
		print("AutoIndicatorStop: Aktiviert.")
	else
		print("AutoIndicatorStop ist bereits aktiviert.")
	end
end

function ConsoleCommands:consoleCommandDeactivateAIS()
	if AISMod.ENABLE_AIS then
		AISMod.ENABLE_AIS = false
		print("AutoIndicatorStop: Deaktiviert.")
	else
		print("AutoIndicatorStop ist bereits deaktiviert.")
	end
end

function ConsoleCommands:consoleCommandStatus()
	local status = self.parent.ENABLE_AIS and "aktiv" or "deaktiv"
	print("AutoIndicatorStop: " .. status)
end
